#==============================================================================
# Script:   texteToCesana.pl
# Auteur:   IDL
# Objectif:  conversion d'un texte en format cesAna avec lemmes et catgories
#==============================================================================


#==================================== modules externes
use IO::handle;			# import d'une librairie pour le traitement des E/S
OUT->autoflush(1);		# permet de vider le tampons d'criture  mesure qu'on crit dans OUT
STDOUT->autoflush(1);	# permet de vider le tampons d'criture  mesure qu'on crit  l'cran

#==================================== dclaration des variables globales
our $abrev="etc|chap|introd|art|adj|adv|apr|J\\.\\-C|art\\-|av|bibliogr|boul|bull|c\\.\\-\\-d|cap|cf|Cf|conf|chap|coll|dp|d|dit|env|etc|tym|ex|fasc|fig|hab|ibid|id|i\\.e|i|e|ill|inf|introd|ital|loc|cit|M|MM|math|ms|N|N\\.B|N\\.\\-D|N\\.D\\.A\\.|N\\.D\\.E|N\\.D\\.L\\.R|obs|op|cit|ouvr|cit|P\\.C\\.C|p|ex|p|pp|paragr|p|i|pl|p\\.\\-s|Q\\.G|R\\.P|S\\.A|sq|sqq|subst|suiv|sup|suppl|S\\.V\\.P|t|trad|var|vol|zool"; # ne pas oublier d'chapper les points et les tirets

# dfinition des motifs pour la tokenisation (attention  doubler tous les \, pour ne pas qu'ils disparaissent de la chane de car.)
our $motif_nombre="(((\\-\\+)\\s)?(\\d+)((\\,|\\s)\\d\\d\\d)*((\\.|\\,)\\d+)?)";
our $motif_ponctuation="(\\.\\.\\.|[,;:.?!()[\\]{}\"'])"; #  l'intrieur des crochets les seuls mta-car. sont ] et -
our $motif_email="([\\w\\-]+(\\.[\\w\\-]+)*@[\\w\\-]+(\\.[\\w\\-]+)+)";
our $motif_url="((http:\\/\\/)?www(\\.[\\w\\-]+)+((\\/[\\w\\-]+)+\\.[\\w\\-]+)?)";
our $motif_mot="([\\wǟ]+)"; # attention \w n'inclut pas les car accentu dans ActivePerl
# on stocke les motifs dans une liste (ainsi que les types correspondants) afin de les traiter dans une boucle for
our @motifs=($motif_email,$motif_url,$motif_nombre,$motif_ponctuation,$motif_mot);
# attention : l'ordre des motifs n'est pas indiffrents (les plus contraints et moins ambigus d'abord).

our %dico;

#************************************ programme principal 

extractCooc("test.txt","arbre.?","abeille.?",5);

#************************************ sous programmes 


#===================================== extractCooc()

# Fonction de segmentation rversible en phrases
# Entres : 
# 	* arg1 : le nom du fichier en lecture
# 	* arg2, arg3 : les formes recherchs (motifs)
#   * arg4 : la distance maximale de cooccurrence

# Sorties :
# 	* criture du fichier HTML donnant la liste des phrases cooccurrentes

sub extractCooc {
	my ($fileIn,$forme1,$forme2,$distance)=@_;
	my $IN;
	my $OUT;
	my ($nbPhrases, $line, $phrase, @tokens,$token,$f1,$f2,$f12,$trouve1,$trouve2);
	
	open($IN,$fileIn);
	open($OUT,">cooc.html");
	
	print $OUT "<html>\n"; 
	print $OUT "<head><title>Cooccurrences des formes $forme1 et $forme2 dans le fichier $fileIn</title>\n";
	print $OUT "<style>\n";
	print $OUT ".phrase { color:black;font-size: 16px ; background-color:#EEEEEE; }\n";
	print $OUT ".mot {background-color: blue; color: yellow; font-weight:bold; }\n";
	print $OUT "</style>\n";
	print $OUT "</head>\n";
	print $OUT "<body>\n";
	print $OUT "<h1>Cooccurrences extraites du fichier : $fileIn</h1>\n<hr/>\n";

	while ($line=<$IN>) {
		my @phrases=texteToPhrases($line);
		foreach $phrase (@phrases) {
			$nbPhrases++;
			@tokens = texteToListeTok($phrase);
			if (@tokens==0) { next; }
			
			# calcul des frquences
			$trouve1=0;
			$trouve2=0;
			foreach $token (@tokens) {
				if ($token=~/^$forme1$/) { $f1++; $trouve1=1; } ;
				if ($token=~/^$forme2$/) { $f2++; $trouve2=1; } ;
			}
			
			# si l'une des deux formes n'apparait pas on passe  la phrase suivante
			if (!$trouve1 || !$trouve2) { next; }

			# recherche des cooccurrences
			my ($i,$j)=(0,0);
			# recherche de la premire occurrence de $forme1
			while ($i<=$#tokens && $tokens[$i] !~/^$forme1$/ ) { 
				$i++;
			}
			if ($i!=$#tokens) {
				$j=$i+1;
				# recherche de la prochaine occurrence de $forme2
				while ($j<=$#tokens && $tokens[$j] !~/^$forme2$/ ) { 
					# si $forme1 apparat ensuite, mise  jour de $i
					if ($tokens[$j] =~/^$forme1$/) { $i=$j; }
					$j++;
				}
				# cas o une cooccurrence a t trouve
				if ($j<=$#tokens && $tokens[$j]=~/^$forme2$/) { 
					if ($j-$i <=$distance) {
						$f12++;
						$avant=join(" ",@tokens[0..$i-1]);
						$entre=join(" ",@tokens[$i+1..$j-1]);
						$apres=join(" ",@tokens[$j+1..$#tokens]);
						
						print $OUT "<p class='phrase'><b>Phrase $nbPhrases : </b>$avant ";
						print $OUT "<span class='mot'>$tokens[$i]</span>";
						print $OUT " $entre ";
						print $OUT "<span class='mot'>$tokens[$j]</span>";
						print $OUT " $apres</span></p>\n";
					}
				}
			}
		}
	}
	print $OUT "<hr/>\n";
	print $OUT "<h2>Frquence de /$forme1/ : $f1<br/>\n";
	print $OUT "Frquence de /$forme2/ : $f2<br/>\n";
	print $OUT "<font color='blue'>Frquence de cooccurrence : $f12</font></h1>\n";
	print $OUT "</body>\n</html>";
	close ($OUT);
	close ($IN);
}


#===================================== texteToPhrases()

# Fonction de segmentation rversible en phrases
# Entres : 
# 	* arg1 : un texte
# Sorties :
# 	* return @phrases une liste des phrases
sub texteToPhrases {
	my $texte=shift (@_);
	my @phrases=();
	my ($tete,$reste,$decoupe);
	chomp $texte;
	
	# traitement des exceptions : on neutralise le point par une marque <dot>
	$texte=~s/(\s)($abrev)\./$1$2<dot>/g;	# abrviation (prcde d'un espace)
	$texte=~s/([\s\-][A-Z]h?)\./$1<dot>/g;	# initiale de prnom (prcd d'un espace ou d'un tiret
	$texte=~s/([0-9]+)\./$1<dot>/g;			# nombre suivi d'un point

	
	$decoupe=1;
	while ($decoupe) {
	
		if ($texte=~/^([^:;]+?[\.\?\!][\"\)]?)(\s+[A-Z0-9].*)$/) { 
		# cas gnral point esp maj
		# Ne pas oublier le ^ initial, car le +? n'tant pas 'gourmand' il faut forcer le motif 
		#  commencer au dbut de la phrase
		# Par ailleurs l'expr. commence par [^:;]+? plutt que .+? 
		# car s'il y a un point double dans la phrase, on segmente avant le point (rgle suivante)
			$tete=$1;
			$reste=$2;
		} elsif ($texte=~/^(.+?[:;])(.*)$/) {# points doubles
			$tete=$1;
			$reste=$2;
		} else {
			if ($texte=~/\;/) {
				die $texte;
			}
			$tete=$texte;
			$decoupe=0;
		}
		$tete=~s/<dot>/./g; # rtablissement des points
		if ($tete) {
			push(@phrases,$tete);
		}
		$texte=$reste;
	}
	return @phrases;
}

#===================================== testeTexteToPhrases()

# Fonction pour tester texteToPhrases
sub testeTexteToPhrases {
	my $ligne;
	my @l;
	my $phrase;
	
	open(IN,"test.txt");
	open(OUT,">test2.txt");
	while ($ligne=<IN>) {
		@l=texteToPhrases($ligne);
		foreach $phrase (@l) {
			print OUT $phrase."\n";
			#~ print $phrase."\n";
		}
	}
	close(IN);
	close(OUT);
}

#===================================== texteToListeTok ()

# Fonction effectuant la tokenisation
# Entres : 
# 	* arg1 : un texte
# Sorties :
# 	* return @listeTok une liste de tokens

sub texteToListeTok {
	# lecture de l'argument de la fonction
	my $texte=shift @_;
	my $tok;
	my @listeTok;

		
	while ($texte !~/^\s*$/) { 		# on sort de la boucle quand $texte est vide ou ne contient que des espaces
		if ($texte=~/^\s+(.*)/) { 	# lecture et suppression d'ventuels espaces en dbut de ligne
			$texte=$1;
		}
		my $tokenTrouve=0;			# boolen indiquant si on a trouv un token correspondant  un des motifs du tableau @motifs
		for (my $i=0;$i<=$#motifs;$i++) {
			if ($texte=~/^$motifs[$i]/) {			# les ventuels espaces de tte sont avals  ce niveau !!!
				$tok=$1;							# on enregistre le token
				$texte=substr($texte,length($tok));	# on ne peut utiliser $2, car les motifs contiennent des sous-parties parenthses
				$tokenTrouve=1;
				last;				# on sort de la boucle for (on ne veut reconnatre qu'un seul token  chaque passage)
			}
		}
		if (!$tokenTrouve) {		# si aucun des motifs n'a permis de trouver un token
			$texte=~/^([^\wǟ])(.*)/; # on avale le caractres non-alphabtiques en tte de chane
			$tok=$1;
			$texte=$2;
		}
		push (@listeTok, $tok);
	}
	return (@listeTok);
}
	
#================================================= lireDico()

# Fonction effectuant la lecture du dictionnaire et le remplissage du hash global %dico
# Entres : 
# 	* arg1 : le nom de fichier du dictionnaire  lire
# Sorties :
# 	* remplissage du hash %dico

sub lireDico {
	my $fileDico=shift;
	local *IN;
	
	%dico=();
	
	open(IN,$fileDico);
	while (<IN>) {
		if (/^([^\t]+)\t([^\t]+)\t(.*)/) {
			$forme=$1;
			$lemme=$2;
			$catTraits=$3;
			if (exists($dico{$forme})) {
				push(@{$dico{$forme}},$lemme."\t".$catTraits);
			} else {
				$dico{$forme}=[$lemme."\t".$catTraits];
			}
		}
	}
	close(IN);
}
